@@ -26,4 +26,7 @@
 #define USE_FIELD_10X26 1
 #define USE_SCALAR_8X32 1
 
+#define ECMULT_GEN_PREC_BITS 4
+#define ECMULT_WINDOW_SIZE 15
+
 #endif /* BITCOIN_LIBSECP256K1_CONFIG_H */
