@@ -62,12 +62,8 @@
         <translation>Færslugildi sem þiggja til sín</translation>
     </message>
     <message>
-        <source>These are your Litecoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
-        <translation>Þetta eru Litecoin færslugildin sem senda greiðslur. Skoðið ævinlega vel upphæðina og færslugildin sem þiggja greiðslur áður en mynt er send.</translation>
-    </message>
-    <message>
-        <source>These are your Litecoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Þetta eru Litecoin færslugildin sem þiggja greiðslur. Mælt er með að nota aldrei sama færslugildið til að þiggja fleiri en eina greiðslu.</translation>
+        <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
+        <translation>Þetta eru Bitcoin færslugildin sem senda greiðslur. Skoðið ævinlega vel upphæðina og færslugildin sem þiggja greiðslur áður en mynt er send.</translation>
     </message>
     <message>
         <source>&amp;Copy Address</source>
@@ -131,10 +127,6 @@
         <source>Repeat new passphrase</source>
         <translation>Endurtaktu nýja lykilsetningu</translation>
     </message>
-    <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>Skráðu nýju lykilsetninguna í veskið. &lt;br/&gt;Vinsamlegast notaðu lykilsetningu með &lt;b&gt;tíu eða fleiri slembibókstöfum&lt;/b&gt;, eða &lt;b&gt;átta eða fleiri orðum&lt;/b&gt;.</translation>
-    </message>
     <message>
         <source>Encrypt wallet</source>
         <translation>Dulkóða veski</translation>
@@ -159,17 +151,13 @@
         <source>Change passphrase</source>
         <translation>Breyta lykilsetningu</translation>
     </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Skráðu gömlu lykilsetninguna og þá nýju í veskið.</translation>
-    </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Staðfesta dulkóðun veskis</translation>
     </message>
     <message>
-        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR LITECOINS&lt;/b&gt;!</source>
-        <translation>Viðvörun: Ef þú dulkóðar veskið og týnir lykilsetningunn þá munt þú &lt;b&gt;TAPA ALLRI ÞINNI LITECOIN MYNT&lt;/b&gt;!</translation>
+        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
+        <translation>Viðvörun: Ef þú dulkóðar veskið og týnir lykilsetningunn þá munt þú &lt;b&gt;TAPA ALLRI ÞINNI BITCOIN MYNT&lt;/b&gt;!</translation>
     </message>
     <message>
         <source>Are you sure you wish to encrypt your wallet?</source>
@@ -179,6 +167,10 @@
         <source>Wallet encrypted</source>
         <translation>Veski dulkóðað</translation>
     </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation>Veski sem á að dulkóða</translation>
+    </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
         <translation>MIKILVÆGT: Nýja dulkóðaða veskisskráin þarf að koma í staðinn fyrir öll fyrri afrit sem þú hefur gert af upprunalegu veskisskránni. Af öryggisástæðum munu öll fyrri afrit af ódulkóðaða veskinu verða óvirk um leið og þú byrjar að nota nýja, dulkóðaða veskið.</translation>
@@ -322,8 +314,8 @@
         <translation>Endurraða blokkum á drifi...</translation>
     </message>
     <message>
-        <source>Send coins to a Litecoin address</source>
-        <translation>Senda mynt í Litecoin færslugildi</translation>
+        <source>Send coins to a Bitcoin address</source>
+        <translation>Senda mynt í Bitcoin færslugildi</translation>
     </message>
     <message>
         <source>Backup wallet to another location</source>
@@ -333,22 +325,10 @@
         <source>Change the passphrase used for wallet encryption</source>
         <translation>Breyta lykilsetningunni sem gildir um dulkóðun veskis</translation>
     </message>
-    <message>
-        <source>&amp;Debug window</source>
-        <translation>&amp;Kembunargluggi</translation>
-    </message>
-    <message>
-        <source>Open debugging and diagnostic console</source>
-        <translation>Opna kembunar- og greiningarstjórnborð</translation>
-    </message>
     <message>
         <source>&amp;Verify message...</source>
         <translation>&amp;Yfirfara skilaboð...</translation>
     </message>
-    <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Senda</translation>
@@ -370,12 +350,12 @@
         <translation>Dulkóða einkalyklana sem tilheyra veskinu þínu</translation>
     </message>
     <message>
-        <source>Sign messages with your Litecoin addresses to prove you own them</source>
-        <translation>Kvitta undir skilaboð með Litecoin færslugildunum þínum til að sanna að þú eigir þau</translation>
+        <source>Sign messages with your Bitcoin addresses to prove you own them</source>
+        <translation>Kvitta undir skilaboð með Bitcoin færslugildunum þínum til að sanna að þú eigir þau</translation>
     </message>
     <message>
-        <source>Verify messages to ensure they were signed with specified Litecoin addresses</source>
-        <translation>Yfirfara skilaboð til að tryggja að kvittað hafi verið fyrir þau með tilteknum Litecoin færslugildum</translation>
+        <source>Verify messages to ensure they were signed with specified Bitcoin addresses</source>
+        <translation>Yfirfara skilaboð til að tryggja að kvittað hafi verið fyrir þau með tilteknum Bitcoin færslugildum</translation>
     </message>
     <message>
         <source>&amp;File</source>
@@ -394,8 +374,8 @@
         <translation>Tólaborð flipa</translation>
     </message>
     <message>
-        <source>Request payments (generates QR codes and litecoin: URIs)</source>
-        <translation>Óska eftir greiðslum (býr til QR kóða og litecoin: URI)</translation>
+        <source>Request payments (generates QR codes and bitcoin: URIs)</source>
+        <translation>Óska eftir greiðslum (býr til QR kóða og bitcoin: URI)</translation>
     </message>
     <message>
         <source>Show the list of used sending addresses and labels</source>
@@ -406,8 +386,8 @@
         <translation>Sýna færslugildi sem notuð hafa verið til að taka við mynt og merkingar þeirra</translation>
     </message>
     <message>
-        <source>Open a litecoin: URI or payment request</source>
-        <translation>Opna litecoin: URI eða greiðslubeiðni</translation>
+        <source>&amp;Command-line options</source>
+        <translation>&amp;Valkostir skipanalínu</translation>
     </message>
     <message>
         <source>Indexing blocks on disk...</source>
@@ -446,8 +426,8 @@
         <translation>Uppfært</translation>
     </message>
     <message>
-        <source>Show the %1 help message to get a list with possible Litecoin command-line options</source>
-        <translation>Sýna %1 hjálparskilaboðin til að fá lista yfir valkosti Litecoin aðgerðir í skipanalínu</translation>
+        <source>Show the %1 help message to get a list with possible Bitcoin command-line options</source>
+        <translation>Sýna %1 hjálparskilaboðin til að fá lista yfir valkosti Bitcoin aðgerðir í skipanalínu</translation>
     </message>
     <message>
         <source>%1 client</source>
@@ -515,11 +495,7 @@
         <source>Wallet is &lt;b&gt;encrypted&lt;/b&gt; and currently &lt;b&gt;locked&lt;/b&gt;</source>
         <translation>Veskið er &lt;b&gt;dulkóðað&lt;/b&gt; and currently &lt;b&gt;locked&lt;/b&gt;</translation>
     </message>
-    <message>
-        <source>A fatal error occurred. Litecoin can no longer continue safely and will quit.</source>
-        <translation>Alvarleg villa átti sér stað. Litecoin getur ekki haldið áfram með öruggum hætti og stoppar hér.</translation>
-    </message>
-</context>
+    </context>
 <context>
     <name>CoinControlDialog</name>
     <message>
@@ -595,6 +571,12 @@
         <translation>(ekkert merki)</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -630,8 +612,8 @@
         <translation>Breyta sendingarfærslugildi</translation>
     </message>
     <message>
-        <source>The entered address "%1" is not a valid Litecoin address.</source>
-        <translation>Færslugildið sem slegið var inn "%1" er ekki leyfilegt Litecoin færslugildi.</translation>
+        <source>The entered address "%1" is not a valid Bitcoin address.</source>
+        <translation>Færslugildið sem slegið var inn "%1" er ekki leyfilegt Bitcoin færslugildi.</translation>
     </message>
     </context>
 <context>
@@ -643,8 +625,8 @@
 <context>
     <name>Intro</name>
     <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
+        <source>Bitcoin</source>
+        <translation>Bitcoin</translation>
     </message>
     <message>
         <source>Error</source>
@@ -665,6 +647,9 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -706,7 +691,10 @@
         <source>Current total balance in watch-only addresses</source>
         <translation>Innistæða á færslugildum sem eru einungis til skoðunar</translation>
     </message>
-</context>
+    </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    </context>
 <context>
     <name>PaymentServer</name>
     <message>
@@ -724,11 +712,12 @@
         <translation>Upphæð</translation>
     </message>
     </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>Resulting URI too long, try to reduce the text for label / message.</source>
+        <translation>URI varð of langt, reyndu að minnka texta í merki / skilaboðum.</translation>
+    </message>
     </context>
 <context>
     <name>RPCConsole</name>
@@ -736,10 +725,6 @@
         <source>Block chain</source>
         <translation>Blokkarkeðja</translation>
     </message>
-    <message>
-        <source>Current number of blocks</source>
-        <translation>Núverandi fjöldi blokka</translation>
-    </message>
     <message>
         <source>Starting Block</source>
         <translation>Upphafsblokk</translation>
@@ -771,24 +756,8 @@
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
-        <source>Address</source>
-        <translation>Vistfang</translation>
-    </message>
-    <message>
-        <source>Amount</source>
-        <translation>Upphæð</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Merki</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Veski</translation>
-    </message>
-    <message>
-        <source>Resulting URI too long, try to reduce the text for label / message.</source>
-        <translation>URI varð of langt, reyndu að minnka texta í merki / skilaboðum.</translation>
+        <source>Amount:</source>
+        <translation>Upphæð:</translation>
     </message>
     </context>
 <context>
@@ -848,18 +817,12 @@
         <translation>&amp;Merki:</translation>
     </message>
     </context>
-<context>
-    <name>SendConfirmationDialog</name>
-    </context>
 <context>
     <name>ShutdownWindow</name>
     </context>
 <context>
     <name>SignVerifyMessageDialog</name>
     </context>
-<context>
-    <name>SplashScreen</name>
-    </context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
@@ -945,6 +908,10 @@
         <source>Export the data in the current tab to a file</source>
         <translation>Flytja gögn í flipanum í skrá</translation>
     </message>
+    <message>
+        <source>Error</source>
+        <translation>Villa</translation>
+    </message>
     </context>
 <context>
     <name>bitcoin-core</name>
@@ -956,17 +923,5 @@
         <source>Warning: The network does not appear to fully agree! Some miners appear to be experiencing issues.</source>
         <translation>Viðvörun: Netið er ekki í fullu samræmi! Einhver námuvinnsla virðist í ólagi.</translation>
     </message>
-    <message>
-        <source>Information</source>
-        <translation>Upplýsingar</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>Viðvörun</translation>
-    </message>
-    <message>
-        <source>Error</source>
-        <translation>Villa</translation>
-    </message>
-</context>
+    </context>
 </TS>
\ No newline at end of file
