@@ -3,11 +3,11 @@
     <name>AddressBookPage</name>
     <message>
         <source>Right-click to edit address or label</source>
-        <translation>చిరునామా లేదా లేబుల్ సవరించడానికి రైట్-క్లిక్ చేయండి</translation>
+        <translation>చిరునామా లేదా లేబుల్ సవరించు -క్లిక్ చేయండి</translation>
     </message>
     <message>
         <source>Create a new address</source>
-        <translation>క్రొత్త చిరునామా సృష్టించండి</translation>
+        <translation>క్రొత్త చిరునామా సృష్టించు</translation>
     </message>
     <message>
         <source>&amp;New</source>
@@ -29,6 +29,14 @@
         <source>Delete the currently selected address from the list</source>
         <translation>ప్రస్తుతం ఎంచుకున్న చిరునామా ను జాబితా నుండి తీసివేయండి</translation>
     </message>
+    <message>
+        <source>Enter address or label to search</source>
+        <translation>చిరునామా లేదా ఏదైనా పేరును వెతకండి</translation>
+    </message>
+    <message>
+        <source>Export the data in the current tab to a file</source>
+        <translation>ప్రస్తుతం ఉన్న సమాచారాన్ని ఫైల్ లోనికి ఎగుమతి చేసుకోండి</translation>
+    </message>
     <message>
         <source>&amp;Export</source>
         <translation>ఎగుమతి చేయండి</translation>
@@ -41,6 +49,10 @@
         <source>Choose the address to send coins to</source>
         <translation>కోయిన్స్ పంపుటకు చిరునామా ను ఎంచుకోండి</translation>
     </message>
+    <message>
+        <source>Choose the address to receive coins with</source>
+        <translation>నాణెం అందుకోవటానికి చిరునామాను ఎంచుకోండి</translation>
+    </message>
     <message>
         <source>C&amp;hoose</source>
         <translation>ఎంచుకోండి</translation>
@@ -54,12 +66,16 @@
         <translation>అందుకునే చిరునామాలు</translation>
     </message>
     <message>
-        <source>These are your Litecoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
+        <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation>ఇవి మీరు పంపే చెల్లింపుల బిట్‌కాయిన్ చిరునామాలు. నాణేలు పంపే ముందు ప్రతిసారి అందుకునే చిరునామా మరియు చెల్లింపు మొత్తం సరిచూసుకోండి.</translation>
     </message>
     <message>
-        <source>These are your Litecoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>ఇవి మీరు అందుకునే చెల్లింపుల బిట్‌కాయిన్ చిరునామాలు. ప్రతీ లావాదేవీకి క్రొత్త అందుకునే చిరునామా వాడటం మంచిది.</translation>
+        <source>&amp;Copy Address</source>
+        <translation>చిరునామాను కాపీ చెయ్యండి</translation>
+    </message>
+    <message>
+        <source>Copy &amp;Label</source>
+        <translation>కాపీ &amp; ఉల్లాకు</translation>
     </message>
     <message>
         <source>&amp;Edit</source>
@@ -69,11 +85,19 @@
         <source>Export Address List</source>
         <translation>చిరునామా జాబితాను ఎగుమతి చేయండి</translation>
     </message>
+    <message>
+        <source>Comma separated file (*.csv)</source>
+        <translation>కోమా వల్ల విభజించిన ఫైల్ (*.csv)</translation>
+    </message>
     <message>
         <source>Exporting Failed</source>
         <translation>ఎగుమతి విఫలమయ్యింది</translation>
     </message>
-    </context>
+    <message>
+        <source>There was an error trying to save the address list to %1. Please try again.</source>
+        <translation>చిరునామా పట్టికను %1 లోనికి ప్రోదుపరుచుటలో లోపము. మరుల ప్రయత్నించి చుడండి.</translation>
+    </message>
+</context>
 <context>
     <name>AddressTableModel</name>
     <message>
@@ -84,7 +108,11 @@
         <source>Address</source>
         <translation>చిరునామా</translation>
     </message>
-    </context>
+    <message>
+        <source>(no label)</source>
+        <translation>( ఉల్లాకు లేదు )</translation>
+    </message>
+</context>
 <context>
     <name>AskPassphraseDialog</name>
     <message>
@@ -104,15 +132,39 @@
         <translation>క్రొత్త సంకేతపదము మరలా ఇవ్వండి</translation>
     </message>
     <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>జోలెకు క్రొత్త సంకేతపదము ఇవ్వండి.&lt;br/&gt; &lt;b&gt;పది లేదా ఎక్కువ యాదృచ్ఛిక అక్షరాలు&lt;/b&gt;, లేక &lt;b&gt;ఎనిమిది కంటే ఎక్కువ పదాలు&lt;/b&gt; కలిగి ఉన్న సంకేతపదము దయచేసి వాడండి.</translation>
+        <source>Show passphrase</source>
+        <translation>సంకేతపదమును చూపించు</translation>
+    </message>
+    <message>
+        <source>Encrypt wallet</source>
+        <translation>వాలెట్‌ను గుప్తీకరించండి</translation>
+    </message>
+    <message>
+        <source>This operation needs your wallet passphrase to unlock the wallet.</source>
+        <translation>ఈ ఆపరేషన్‌కు వాలెట్‌ను అన్‌లాక్ చేయడానికి మీ వాలెట్ పాస్‌ఫ్రేజ్ అవసరం.</translation>
     </message>
     <message>
         <source>Unlock wallet</source>
         <translation>వాలెట్ అన్లాక్</translation>
     </message>
     <message>
-        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR LITECOINS&lt;/b&gt;!</source>
+        <source>This operation needs your wallet passphrase to decrypt the wallet.</source>
+        <translation>ఈ ఆపరేషన్‌కు వాలెట్‌ను డీక్రిప్ట్ చేయడానికి మీ వాలెట్ పాస్‌ఫ్రేజ్ అవసరం.</translation>
+    </message>
+    <message>
+        <source>Decrypt wallet</source>
+        <translation>డీక్రిప్ట్ వాలెట్</translation>
+    </message>
+    <message>
+        <source>Change passphrase</source>
+        <translation>పాస్‌ఫ్రేజ్‌ని మార్చండి</translation>
+    </message>
+    <message>
+        <source>Confirm wallet encryption</source>
+        <translation>వాలెట్ గుప్తీకరణను నిర్ధారించండి</translation>
+    </message>
+    <message>
+        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
         <translation>హెచ్చరిక: మీ జోలెని సంకేతపరిచి మీ సంకేతపదము కోల్పోతే, &lt;b&gt;మీ బిట్‌కాయిన్లు అన్నీ కోల్పోతారు&lt;/b&gt;</translation>
     </message>
     <message>
@@ -123,23 +175,75 @@
         <source>Wallet encrypted</source>
         <translation>జోలె సంకేతపరబడింది</translation>
     </message>
+    <message>
+        <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
+        <translation>వాలెట్ కోసం క్రొత్త పాస్‌ఫ్రేజ్‌ని నమోదు చేయండి.&lt;br/&gt; దయచేసి &lt;b&gt;పది లేదా అంతకంటే ఎక్కువ యాదృచ్ఛిక అక్షరాల&lt;/b&gt; పాస్‌ఫ్రేజ్‌ని లేదా &lt;b&gt;ఎనిమిది లేదా అంతకంటే ఎక్కువ పదాలను ఉపయోగించండి.&lt;/b&gt;</translation>
+    </message>
+    <message>
+        <source>Enter the old passphrase and new passphrase for the wallet.</source>
+        <translation>Enter the old passphrase and new passphrase for the wallet.</translation>
+    </message>
+    <message>
+        <source>Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
+        <translation>మీ వాలెట్‌ను గుప్తీకరించడం వల్ల మీ కంప్యూటర్‌కు హాని కలిగించే మాల్వేర్ దొంగిలించకుండా మీ బిట్‌కాయిన్‌లను పూర్తిగా రక్షించలేమని గుర్తుంచుకోండి.</translation>
+    </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation>ఎన్క్రిప్ట్ చేయవలసిన వాలెట్</translation>
+    </message>
+    <message>
+        <source>Your wallet is about to be encrypted. </source>
+        <translation>మీ వాలెట్ గుప్తీకరించబోతోంది.</translation>
+    </message>
+    <message>
+        <source>Your wallet is now encrypted. </source>
+        <translation>cheraveyu chirunama</translation>
+    </message>
+    <message>
+        <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
+        <translation>ముఖ్యమైనది: మీరు మీ వాలెట్ ఫైల్‌తో చేసిన మునుపటి బ్యాకప్‌లను కొత్తగా రూపొందించిన, గుప్తీకరించిన వాలెట్ ఫైల్‌తో భర్తీ చేయాలి. భద్రతా కారణాల దృష్ట్యా, మీరు క్రొత్త, గుప్తీకరించిన వాలెట్ ఉపయోగించడం ప్రారంభించిన వెంటనే గుప్తీకరించని వాలెట్ ఫైల్ యొక్క మునుపటి బ్యాకప్‌లు నిరుపయోగంగా మారతాయి.</translation>
+    </message>
     <message>
         <source>Wallet encryption failed</source>
         <translation>జోలె సంకేతపరచడం విఫలమయ్యింది</translation>
     </message>
+    <message>
+        <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
+        <translation>lopali tappidam valla mee yokka wallet encryption samapthamu avaledu</translation>
+    </message>
+    <message>
+        <source>The supplied passphrases do not match.</source>
+        <translation>సరఫరా చేసిన పాస్‌ఫ్రేజ్‌లు సరిపోలడం లేదు.</translation>
+    </message>
     </context>
 <context>
     <name>BanTableModel</name>
     </context>
 <context>
     <name>BitcoinGUI</name>
+    <message>
+        <source>Sign &amp;message...</source>
+        <translation>సంతకము మరియు సమాచారం</translation>
+    </message>
+    <message>
+        <source>Synchronizing with network...</source>
+        <translation>సమూహము తో సమకాలీకరణ చేయబడినది</translation>
+    </message>
+    <message>
+        <source>&amp;Overview</source>
+        <translation>&amp;అవలోకనం</translation>
+    </message>
     <message>
         <source>E&amp;xit</source>
         <translation>నిష్క్రమించు</translation>
     </message>
     <message>
-        <source>Litecoin</source>
-        <translation>బిట్కోయిన్</translation>
+        <source>Create a new wallet</source>
+        <translation>&lt;div&gt;&lt;/div&gt;</translation>
+    </message>
+    <message>
+        <source>Wallet:</source>
+        <translation>ధనమును తీసుకొనిపోవు సంచి</translation>
     </message>
     <message>
         <source>&amp;Send</source>
@@ -176,10 +280,24 @@
         <source>Quantity:</source>
         <translation>పరిమాణం</translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>మొత్తం</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation>తేదీ</translation>
     </message>
+    <message>
+        <source>(no label)</source>
+        <translation>( ఉల్లాకు లేదు )</translation>
+    </message>
+    </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
     </context>
 <context>
     <name>EditAddressDialog</name>
@@ -193,7 +311,7 @@
 <context>
     <name>Intro</name>
     <message>
-        <source>Litecoin</source>
+        <source>Bitcoin</source>
         <translation>బిట్కోయిన్</translation>
     </message>
     <message>
@@ -207,6 +325,9 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -217,6 +338,9 @@
 <context>
     <name>OverviewPage</name>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    </context>
 <context>
     <name>PaymentServer</name>
     </context>
@@ -225,14 +349,15 @@
     </context>
 <context>
     <name>QObject</name>
+    <message>
+        <source>Amount</source>
+        <translation>మొత్తం</translation>
+    </message>
     <message>
         <source>unknown</source>
         <translation>తెలియదు</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
     </context>
@@ -245,20 +370,8 @@
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
-        <source>Address</source>
-        <translation>చిరునామా</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>ఉల్లాకు</translation>
-    </message>
-    <message>
-        <source>Message</source>
-        <translation>సందేశం</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>వాలెట్</translation>
+        <source>Wallet:</source>
+        <translation>ధనమును తీసుకొనిపోవు సంచి</translation>
     </message>
     </context>
 <context>
@@ -275,6 +388,10 @@
         <source>Message</source>
         <translation>సందేశం</translation>
     </message>
+    <message>
+        <source>(no label)</source>
+        <translation>( ఉల్లాకు లేదు )</translation>
+    </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
@@ -282,22 +399,20 @@
         <source>Quantity:</source>
         <translation>పరిమాణం</translation>
     </message>
-    </context>
+    <message>
+        <source>(no label)</source>
+        <translation>( ఉల్లాకు లేదు )</translation>
+    </message>
+</context>
 <context>
     <name>SendCoinsEntry</name>
     </context>
-<context>
-    <name>SendConfirmationDialog</name>
-    </context>
 <context>
     <name>ShutdownWindow</name>
     </context>
 <context>
     <name>SignVerifyMessageDialog</name>
     </context>
-<context>
-    <name>SplashScreen</name>
-    </context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
@@ -331,6 +446,10 @@
         <source>Merchant</source>
         <translation>వర్తకుడు</translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>మొత్తం</translation>
+    </message>
     </context>
 <context>
     <name>TransactionDescDialog</name>
@@ -345,9 +464,17 @@
         <source>Label</source>
         <translation>ఉల్లాకు</translation>
     </message>
+    <message>
+        <source>(no label)</source>
+        <translation>( ఉల్లాకు లేదు )</translation>
+    </message>
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>Comma separated file (*.csv)</source>
+        <translation>కోమా వల్ల విభజించిన ఫైల్ (*.csv)</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation>తేదీ</translation>
@@ -377,7 +504,11 @@
     </context>
 <context>
     <name>WalletFrame</name>
-    </context>
+    <message>
+        <source>Create a new wallet</source>
+        <translation>&lt;div&gt;&lt;/div&gt;</translation>
+    </message>
+</context>
 <context>
     <name>WalletModel</name>
     </context>
@@ -387,24 +518,16 @@
         <source>&amp;Export</source>
         <translation>ఎగుమతి చేయండి</translation>
     </message>
-    </context>
-<context>
-    <name>bitcoin-core</name>
     <message>
-        <source>Litecoin Core</source>
-        <translation>బిట్కోయిన్ కోర్</translation>
-    </message>
-    <message>
-        <source>Information</source>
-        <translation>వర్తమానము</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>హెచ్చరిక</translation>
+        <source>Export the data in the current tab to a file</source>
+        <translation>ప్రస్తుతం ఉన్న సమాచారాన్ని ఫైల్ లోనికి ఎగుమతి చేసుకోండి</translation>
     </message>
     <message>
         <source>Error</source>
         <translation>లోపం</translation>
     </message>
-</context>
+    </context>
+<context>
+    <name>bitcoin-core</name>
+    </context>
 </TS>
\ No newline at end of file
