@@ -1,17 +1,21 @@
-// Copyright (c) 2011-2018 The Bitcoin Core developers
+// Copyright (c) 2011-2019 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #ifndef BITCOIN_QT_SENDCOINSENTRY_H
 #define BITCOIN_QT_SENDCOINSENTRY_H
 
-#include <qt/walletmodel.h>
+#include <qt/sendcoinsrecipient.h>
 
 #include <QStackedWidget>
 
 class WalletModel;
 class PlatformStyle;
 
+namespace interfaces {
+class Node;
+} // namespace interfaces
+
 namespace Ui {
     class SendCoinsEntry;
 }
@@ -40,6 +44,9 @@ public:
     void setAddress(const QString &address);
     void setAmount(const CAmount &amount);
 
+    void setPegInAddress(const std::string& address);
+    void setPegOut(const bool pegout_set);
+
     /** Set up the tab chain manually, as Qt messes up the tab chain by default in some cases
      *  (issue https://bugreports.qt-project.org/browse/QTBUG-10907).
      */
@@ -70,6 +77,8 @@ private:
     Ui::SendCoinsEntry *ui;
     WalletModel *model;
     const PlatformStyle *platformStyle;
+    std::string pegInAddress;
+    bool pegout;
 
     bool updateLabel(const QString &address);
 };
