@@ -5,13 +5,10 @@
 #ifndef BITCOIN_TEST_FUZZ_FUZZ_H
 #define BITCOIN_TEST_FUZZ_FUZZ_H
 
-#include <functional>
 #include <stdint.h>
 #include <vector>
 
-
-const std::function<std::string(const char*)> G_TRANSLATION_FUN = nullptr;
-
-void test_one_input(std::vector<uint8_t> buffer);
+void initialize();
+void test_one_input(const std::vector<uint8_t>& buffer);
 
 #endif // BITCOIN_TEST_FUZZ_FUZZ_H
