@@ -1,9 +1,8 @@
 #include <boost/test/unit_test.hpp>
 
-#include <uint256.h>
-#include <util.h>
-#include <utilstrencodings.h>
 #include <crypto/scrypt.h>
+#include <uint256.h>
+#include <util/strencodings.h>
 
 BOOST_AUTO_TEST_SUITE(scrypt_tests)
 
