@@ -1,4 +1,7 @@
 #!/usr/bin/env python3
+# Copyright (c) 2017-2019 The Bitcoin Core developers
+# Distributed under the MIT software license, see the accompanying
+# file COPYING or http://www.opensource.org/licenses/mit-license.php.
 """Combine logs from multiple bitcoin nodes as well as the test_framework log.
 
 This streams the combined log output to stdout. Use combine_logs.py > outputfile
@@ -11,6 +14,7 @@ from collections import defaultdict, namedtuple
 import heapq
 import itertools
 import os
+import pathlib
 import re
 import sys
 import tempfile
@@ -51,9 +55,23 @@ def main():
     if not args.testdir:
         print("Opening latest test directory: {}".format(testdir), file=sys.stderr)
 
+    colors = defaultdict(lambda: '')
+    if args.color:
+        colors["test"] = "\033[0;36m"  # CYAN
+        colors["node0"] = "\033[0;34m"  # BLUE
+        colors["node1"] = "\033[0;32m"  # GREEN
+        colors["node2"] = "\033[0;31m"  # RED
+        colors["node3"] = "\033[0;33m"  # YELLOW
+        colors["reset"] = "\033[0m"  # Reset font color
+
     log_events = read_logs(testdir)
 
-    print_logs(log_events, color=args.color, html=args.html)
+    if args.html:
+        print_logs_html(log_events)
+    else:
+        print_logs_plain(log_events, colors)
+        print_node_warnings(testdir, colors)
+
 
 def read_logs(tmp_dir):
     """Reads log files.
@@ -61,9 +79,18 @@ def read_logs(tmp_dir):
     Delegates to generator function get_log_events() to provide individual log events
     for each of the input log files."""
 
+    # Find out what the folder is called that holds the debug.log file
+    glob = pathlib.Path(tmp_dir).glob('node0/**/debug.log')
+    path = next(glob, None)
+    if path:
+        assert next(glob, None) is None #  more than one debug.log, should never happen
+        chain = re.search(r'node0/(.+?)/debug\.log$', path.as_posix()).group(1)  # extract the chain name
+    else:
+        chain = 'regtest'  # fallback to regtest (should only happen when none exists)
+
     files = [("test", "%s/test_framework.log" % tmp_dir)]
     for i in itertools.count():
-        logfile = "{}/node{}/regtest/debug.log".format(tmp_dir, i)
+        logfile = "{}/node{}/{}/debug.log".format(tmp_dir, i, chain)
         if not os.path.isfile(logfile):
             break
         files.append(("node%d" % i, logfile))
@@ -71,6 +98,26 @@ def read_logs(tmp_dir):
     return heapq.merge(*[get_log_events(source, f) for source, f in files])
 
 
+def print_node_warnings(tmp_dir, colors):
+    """Print nodes' errors and warnings"""
+
+    warnings = []
+    for stream in ['stdout', 'stderr']:
+        for i in itertools.count():
+            folder = "{}/node{}/{}".format(tmp_dir, i, stream)
+            if not os.path.isdir(folder):
+                break
+            for (_, _, fns) in os.walk(folder):
+                for fn in fns:
+                    warning = pathlib.Path('{}/{}'.format(folder, fn)).read_text().strip()
+                    if warning:
+                        warnings.append(("node{} {}".format(i, stream), warning))
+
+    print()
+    for w in warnings:
+        print("{} {} {} {}".format(colors[w[0].split()[0]], w[0], w[1], colors["reset"]))
+
+
 def find_latest_test_dir():
     """Returns the latest tmpfile test directory prefix."""
     tmpdir = tempfile.gettempdir()
@@ -127,34 +174,28 @@ def get_log_events(source, logfile):
     except FileNotFoundError:
         print("File %s could not be opened. Continuing without it." % logfile, file=sys.stderr)
 
-def print_logs(log_events, color=False, html=False):
-    """Renders the iterator of log events into text or html."""
-    if not html:
-        colors = defaultdict(lambda: '')
-        if color:
-            colors["test"] = "\033[0;36m"   # CYAN
-            colors["node0"] = "\033[0;34m"  # BLUE
-            colors["node1"] = "\033[0;32m"  # GREEN
-            colors["node2"] = "\033[0;31m"  # RED
-            colors["node3"] = "\033[0;33m"  # YELLOW
-            colors["reset"] = "\033[0m"     # Reset font color
 
-        for event in log_events:
-            lines = event.event.splitlines()
-            print("{0} {1: <5} {2} {3}".format(colors[event.source.rstrip()], event.source, lines[0], colors["reset"]))
-            if len(lines) > 1:
-                for line in lines[1:]:
-                    print("{0}{1}{2}".format(colors[event.source.rstrip()], line, colors["reset"]))
+def print_logs_plain(log_events, colors):
+    """Renders the iterator of log events into text."""
+    for event in log_events:
+        lines = event.event.splitlines()
+        print("{0} {1: <5} {2} {3}".format(colors[event.source.rstrip()], event.source, lines[0], colors["reset"]))
+        if len(lines) > 1:
+            for line in lines[1:]:
+                print("{0}{1}{2}".format(colors[event.source.rstrip()], line, colors["reset"]))
 
-    else:
-        try:
-            import jinja2
-        except ImportError:
-            print("jinja2 not found. Try `pip install jinja2`")
-            sys.exit(1)
-        print(jinja2.Environment(loader=jinja2.FileSystemLoader('./'))
+
+def print_logs_html(log_events):
+    """Renders the iterator of log events into html."""
+    try:
+        import jinja2
+    except ImportError:
+        print("jinja2 not found. Try `pip install jinja2`")
+        sys.exit(1)
+    print(jinja2.Environment(loader=jinja2.FileSystemLoader('./'))
                     .get_template('combined_log_template.html')
                     .render(title="Combined Logs from testcase", log_events=[event._asdict() for event in log_events]))
 
+
 if __name__ == '__main__':
     main()
